/***************************************************************************

    emu.h

    Core header file to be included by most files.

    NOTE: The contents of this file are designed to meet the needs of
    drivers and devices. In addition to this file, you will also need
    to include the headers of any CPUs or other devices that are required.

    If you find yourself needing something outside of this file in a
    driver or device, think carefully about what you are doing.

****************************************************************************

    Copyright Aaron Giles
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are
    met:

        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in
          the documentation and/or other materials provided with the
          distribution.
        * Neither the name 'MAME' nor the names of its contributors may be
          used to endorse or promote products derived from this software
          without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY AARON GILES ''AS IS'' AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL AARON GILES BE LIABLE FOR ANY DIRECT,
    INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
    STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
    IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.

***************************************************************************/

#pragma once

#ifndef __EMU_H__
#define __EMU_H__

// core emulator headers -- must be first
#include "emucore.h"
#include "eminline.h"

// commonly-referenecd utilities imported from lib/util
#include "chd.h"
#include "palette.h"
#include "tagmap.h"
#include "unicode.h"

// emulator-specific utilities
#include "attotime.h"
#include "fileio.h" // remove me once NVRAM is implemented as device
#include "tokenize.h"

// memory and address spaces
#include "memory.h"

// devices and callbacks
#include "devintrf.h"
#include "devcb.h"

// I/O
#include "input.h"
#include "inputseq.h"
#include "inptport.h"
#include "output.h"

// timers, CPU and scheduling
#include "timer.h"
#include "cpuintrf.h"
#include "cpuexec.h"
#include "watchdog.h"

// machine and driver configuration
#include "mconfig.h"
#include "driver.h"

// machine-wide utilities
#include "romload.h"
#include "state.h"

// the running machine
#include "mame.h"

// video-related
#include "drawgfx.h"
#include "tilemap.h"
#include "emupal.h"
#include "video.h"

// sound-related
#include "sound.h"

// generic helpers
#include "drivers/xtal.h"
#include "audio/generic.h"
#include "machine/generic.h"
#include "video/generic.h"

#endif	/* __EMU_H__ */
